//
//  MandelOpperAppDelegate.m
//  MandelOpper
//
//  Created by Ken Anderson on 4/29/12.
//  Copyright (c) 2012 University of Colorado, Boulder. All rights reserved.
//

#import "MandelOpperAppDelegate.h"
#import "Bitmap.h"
#import "BitmapView.h"
#import "CalcOperation.h"

@implementation MandelOpperAppDelegate

@synthesize window = _window;
@synthesize bitmapView = _bitmapView;
@synthesize statusLine = _statusLine;

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
}

- (void) updateStatus: (NSString *) message {
    self.statusLine.stringValue = message;
}

- (void) awakeFromNib {
    [self updateStatus:@"idle"];
    _region = NSMakeRect(-2.0, -1.2, 3.0, 2.4);
}

- (IBAction) start : (id) sender {
    [_queue cancelAllOperations];
    _queue = nil;
    
    [self updateStatus:@"running"];
    
    _queue = [[NSOperationQueue alloc] init];
    _queue.maxConcurrentOperationCount = 100;
    
//    NSLog(@"%ld concurrent operations", (long)_queue.maxConcurrentOperationCount);
    
    NSBlockOperation *allDone = [NSBlockOperation blockOperationWithBlock: ^(void) {
        [self performSelectorOnMainThread:@selector(updateStatus:) withObject:@"done!" waitUntilDone:NO];
    }];
    
    NSRect frame = self.bitmapView.frame;
    NSUInteger height = frame.size.height;
    NSUInteger width = frame.size.width;
    
    _bitmap = [[Bitmap alloc] initWithWidth:width height:height];
    [self.bitmapView setBitmap:_bitmap];
    double deltaY = _region.size.height / height;
    double y = NSMaxY(_region);
    
    for (int i = 0; i < frame.size.height; i++) {
        CalcOperation *op;
        op = [[CalcOperation alloc] initWithBitmap:_bitmap bitmapView:self.bitmapView calculateLine:i xStart:NSMinX(_region) xEnd:NSMaxX(_region) y:y];
        
        [allDone addDependency:op];
        [_queue addOperation:op];
        
        y -= deltaY;
    }
    
    [_queue addOperation:allDone];
    
}

@end
